@@ -1,6 +1,7 @@
 //! Send email through Mailgun
 
 use chrono::prelude::*;
+
 use reqwest;
 use std::collections::HashMap;
 
@@ -20,19 +21,6 @@ impl Default for MessageBody {
     }
 }
 
-impl MessageBody {
-    fn add_to(self, params: &mut HashMap<String, String>) {
-        match self {
-            MessageBody::Html(c) => params.insert(String::from("html"), c),
-            MessageBody::Text(c) => params.insert(String::from("text"), c),
-            MessageBody::HtmlAndText(html, text) => {
-                params.insert(String::from("html"), html);
-                params.insert(String::from("text"), text)
-            }
-        };
-    }
-}
-
 ///! An email to send through Mailgun. Won't send without a body
 #[derive(Default)]
 pub struct Message {
@@ -43,25 +31,32 @@ pub struct Message {
     pub body: MessageBody,
     pub template: Option<String>,
     pub options: Vec<SendOptions>,
+    pub attachments: Vec<Attachment>,
+    pub inline: Vec<Attachment>,
+}
+
+#[derive(Default)]
+pub struct Attachment {
+    pub name: String,
+    pub content: Vec<u8>,
+    pub mime_type: String,
 }
 
 impl Message {
-    fn into_params(self) -> HashMap<String, String> {
+    fn into_params(&self) -> HashMap<String, String> {
         let mut params = HashMap::new();
 
-        Message::add_recipients("to", self.to, &mut params);
-        Message::add_recipients("cc", self.cc, &mut params);
-        Message::add_recipients("bcc", self.bcc, &mut params);
+        Message::add_recipients("to", &self.to, &mut params);
+        Message::add_recipients("cc", &self.cc, &mut params);
+        Message::add_recipients("bcc", &self.bcc, &mut params);
 
-        params.insert(String::from("subject"), self.subject);
+        params.insert(String::from("subject"), self.subject.to_string());
 
-        self.body.add_to(&mut params);
-
-        if let Some(template) = self.template {
+        if let Some(template) = self.template.clone() {
             params.insert("template".to_string(), template);
         }
 
-        for opt in self.options {
+        for opt in &self.options {
             opt.add_to(&mut params);
         }
 
@@ -70,7 +65,7 @@ impl Message {
 
     fn add_recipients(
         field: &str,
-        addresses: Vec<EmailAddress>,
+        addresses: &Vec<EmailAddress>,
         params: &mut HashMap<String, String>,
     ) {
         if !addresses.is_empty() {
@@ -157,9 +152,40 @@ pub fn send_with_request_builder(
     let mut params = msg.into_params();
     params.insert("from".to_string(), sender.to_string());
 
+    let mut form = reqwest::blocking::multipart::Form::new();
+    for (key, value) in params {
+        form = form.text(key, value);
+    }
+    //add message content
+    match msg.body {
+        MessageBody::Text(text) => {
+            form = form.text("text", text);
+        }
+        MessageBody::Html(html) => {
+            form = form.text("html", html);
+        }
+        MessageBody::HtmlAndText(html, text) => {
+            form = form.text("text", text);
+            form = form.text("html", html);
+        }
+    }
+    //add attachments
+    for attachment in msg.attachments {
+        let file_part = reqwest::blocking::multipart::Part::bytes(attachment.content)
+            .file_name(attachment.name.clone())
+            .mime_str(&attachment.mime_type)?;
+        form = form.part("attachment", file_part);
+    }
+    //add inline files
+    for attachment in msg.inline {
+        let file_part = reqwest::blocking::multipart::Part::bytes(attachment.content)
+            .file_name(attachment.name.clone())
+            .mime_str(&attachment.mime_type)?;
+        form = form.part("inline", file_part);
+    }
     let res = request_builder
         .basic_auth("api", Some(creds.api_key.clone()))
-        .form(&params)
+        .multipart(form)
         .send()?
         .error_for_status()?;
 
@@ -173,34 +199,6 @@ mod tests {
     use reqwest::StatusCode;
     use serde_json::json;
 
-    #[test]
-    fn message_body() {
-        let text = Message {
-            body: MessageBody::Text(String::from("hello, world")),
-            ..Default::default()
-        };
-        let params = text.into_params();
-        assert_eq!(params.get("text"), Some(&String::from("hello, world")));
-
-        let html = Message {
-            body: MessageBody::Html(String::from("<body>hello, world</body>")),
-            ..Default::default()
-        };
-        let params = html.into_params();
-        assert_eq!(
-            params.get("html"),
-            Some(&String::from("<body>hello, world</body>"))
-        );
-
-        let both = Message {
-            body: MessageBody::HtmlAndText(String::from("<body/>"), String::from("hello")),
-            ..Default::default()
-        };
-        let params = both.into_params();
-        assert_eq!(params.get("html"), Some(&String::from("<body/>")));
-        assert_eq!(params.get("text"), Some(&String::from("hello")));
-    }
-
     #[test]
     fn message_recipients() {
         let msg = Message {
@@ -285,7 +283,7 @@ mod tests {
         let sender = EmailAddress::name_address("Nick Testla", &format!("mailgun_v3@{}", &domain));
 
         let res = send_email(&creds, &sender, message);
-        assert!(res.is_ok(), format!("{:?}", &res));
+        assert!(res.is_ok(), "{:?}", &res);
     }
 
     #[test]
@@ -323,7 +321,7 @@ mod tests {
         let client = reqwest::blocking::Client::new();
         let request_builder = client.post(&url);
         let res = send_with_request_builder(request_builder, &creds, &sender, message);
-        assert!(res.is_ok(), format!("{:?}", &res));
+        assert!(res.is_ok(), "{:?}", &res);
     }
 }
 
@@ -364,9 +362,41 @@ pub mod async_impl {
         let mut params = msg.into_params();
         params.insert("from".to_string(), sender.to_string());
 
+        let mut form = reqwest::multipart::Form::new();
+        for (key, value) in params {
+            form = form.text(key, value);
+        }
+        //add attachments
+        for attachment in msg.attachments {
+            let file_part = reqwest::multipart::Part::bytes(attachment.content)
+                .file_name(attachment.name.clone())
+                .mime_str(&attachment.mime_type)?;
+            form = form.part("attachment", file_part);
+        }
+        //add inline files
+        for attachment in msg.inline {
+            let file_part = reqwest::multipart::Part::bytes(attachment.content)
+                .file_name(attachment.name.clone())
+                .mime_str(&attachment.mime_type)?;
+            form = form.part("inline", file_part);
+        }
+        //add message content
+        match msg.body {
+            MessageBody::Text(text) => {
+                form = form.text("text", text);
+            }
+            MessageBody::Html(html) => {
+                form = form.text("html", html);
+            }
+            MessageBody::HtmlAndText(html, text) => {
+                form = form.text("text", text);
+                form = form.text("html", html);
+            }
+        }
+
         let res = request_builder
             .basic_auth("api", Some(creds.api_key.clone()))
-            .form(&params)
+            .multipart(form)
             .send()
             .await?
             .error_for_status()?;
@@ -403,7 +433,7 @@ pub mod async_impl {
                 EmailAddress::name_address("Nick Testla", &format!("mailgun_v3@{}", &domain));
 
             let res = send_email(&creds, &sender, message).await;
-            assert!(res.is_ok(), format!("{:?}", &res));
+            assert!(res.is_ok(), "{:?}", &res);
         }
 
         #[ignore]
@@ -418,9 +448,10 @@ pub mod async_impl {
             let message = Message {
                 to: vec![recipient],
                 subject: "Test email".to_string(),
-                options: vec![
-                    SendOptions::Header("X-Mailgun-Variables".to_string(), "{\"test\": \"test\"}".to_string())
-                ],
+                options: vec![SendOptions::Header(
+                    "X-Mailgun-Variables".to_string(),
+                    "{\"test\": \"test\"}".to_string(),
+                )],
                 template: Some("sometest".to_string()),
                 ..Default::default()
             };
@@ -428,7 +459,7 @@ pub mod async_impl {
                 EmailAddress::name_address("Nick Testla", &format!("mailgun_v3@{}", &domain));
 
             let res = send_email(&creds, &sender, message).await;
-            assert!(res.is_ok(), format!("{:?}", &res));
+            assert!(res.is_ok(), "{:?}", &res);
         }
 
         #[tokio::test]
@@ -467,7 +498,7 @@ pub mod async_impl {
             let client = reqwest::Client::new();
             let request_builder = client.post(&url);
             let res = send_with_request_builder(request_builder, &creds, &sender, message).await;
-            assert!(res.is_ok(), format!("{:?}", &res));
+            assert!(res.is_ok(), "{:?}", &res);
         }
     }
 }
